mod migration;
mod migrations;
mod program;
mod sql_json;
mod zone;

pub use migration::*;
pub use migrations::create_migrations;
pub use program::*;

pub use rusqlite::Connection as DbConn;

use sprinklers_core::model::Zones;

use eyre::Result;
use rusqlite::NO_PARAMS;

pub fn setup_db() -> Result<DbConn> {
    // let conn = DbConn::open_in_memory()?;
    let mut conn = DbConn::open("test.db")?;

    // Go ahead and use write ahead log for better perf
    conn.execute_batch("PRAGMA journal_mode=WAL;")?;

    let migs = create_migrations();
    migs.apply(&mut conn)?;

    Ok(conn)
}

pub fn query_zones(conn: &DbConn) -> Result<Zones> {
    let mut statement = conn.prepare_cached(
        "SELECT s.id, s.name, s.interface_id \
        FROM sections AS s;",
    )?;
    let rows = statement.query_map(NO_PARAMS, zone::from_sql)?;
    let mut zones = Zones::new();
    for row in rows {
        let zone = row?;
        zones.insert(zone.id, zone.into());
    }
    Ok(zones)
}
