pub trait CollectionTopics<'t> {
    fn new(prefix: &'t str) -> Self;
    fn ids(&self) -> String;
    fn data(&self, id: u32) -> String;
}

#[derive(Clone, Debug)]
pub struct ZoneTopics<'a>(pub &'a str);

impl<'a> CollectionTopics<'a> for ZoneTopics<'a> {
    fn new(prefix: &'a str) -> Self {
        ZoneTopics(prefix)
    }

    fn ids(&self) -> String {
        // TODO: change nomenclature
        format!("{}/sections", self.0)
    }

    fn data(&self, zone_id: u32) -> String {
        // TODO: change nomenclature
        format!("{}/sections/{}", self.0, zone_id)
    }
}

impl<'a> ZoneTopics<'a> {
    pub fn state(&self, zone_id: u32) -> String {
        // TODO: change nomenclature
        format!("{}/sections/{}/state", self.0, zone_id)
    }
}

#[derive(Clone, Debug)]
pub struct ProgramTopics<'a>(pub &'a str);

impl<'a> CollectionTopics<'a> for ProgramTopics<'a> {
    fn new(prefix: &'a str) -> Self {
        ProgramTopics(prefix)
    }

    fn ids(&self) -> String {
        format!("{}/programs", self.0)
    }

    fn data(&self, zone_id: u32) -> String {
        format!("{}/programs/{}", self.0, zone_id)
    }
}

impl<'a> ProgramTopics<'a> {
    pub fn running(&self, zone_id: u32) -> String {
        format!("{}/programs/{}/running", self.0, zone_id)
    }

    pub fn next_run(&self, zone_id: u32) -> String {
        // TODO: reconcile naming convention
        format!("{}/programs/{}/nextRun", self.0, zone_id)
    }
}

#[derive(Clone, Debug)]
pub struct Topics<T: AsRef<str>>(pub T);

impl<T: AsRef<str>> Topics<T> {
    pub fn new(prefix: T) -> Self {
        Self(prefix)
    }

    pub fn prefix(&self) -> &str {
        self.0.as_ref()
    }

    pub fn connected(&self) -> String {
        format!("{}/connected", self.0.as_ref())
    }

    pub fn zones(&self) -> ZoneTopics {
        ZoneTopics::new(self.0.as_ref())
    }

    pub fn programs(&self) -> ProgramTopics {
        ProgramTopics::new(self.0.as_ref())
    }

    pub fn zone_runner(&self) -> String {
        // TODO: change nomenclature
        format!("{}/section_runner", self.0.as_ref())
    }

    pub fn requests(&self) -> String {
        format!("{}/requests", self.0.as_ref())
    }

    pub fn responses(&self) -> String {
        format!("{}/responses", self.0.as_ref())
    }
}
