use crate::zone_interface::ZoneInterfaceConfig;

use serde::{Deserialize, Serialize};
use tracing::trace;

#[derive(Debug, Serialize, Deserialize)]
#[serde(remote = "sprinklers_mqtt::Options")]
struct MqttOptions {
    pub broker_host: String,
    pub broker_port: u16,
    pub device_id: String,
    pub client_id: String,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Settings {
    #[serde(with = "MqttOptions")]
    pub mqtt: sprinklers_mqtt::Options,
    #[serde(default)]
    pub zone_interface: ZoneInterfaceConfig,
}

impl Settings {
    pub fn new() -> eyre::Result<Self> {
        let mut s = config::Config::new();

        let default_config = config::File::from_str(
            include_str!("../sprinklers_rs.default.json"),
            config::FileFormat::Json,
        );
        s.merge(default_config)?;

        // TODO: specify configuration path from arguments or env
        s.merge(config::File::with_name("sprinklers_rs").required(false))?;

        s.merge(config::Environment::with_prefix("SPRINKLERS").separator("__"))?;

        let settings: Settings = s.try_into()?;

        trace!("settings: {:#?}", settings);

        Ok(settings)
    }
}
