use sprinklers_core::zone_interface::{MockZoneInterface, ZoneInterface, ZoneNum};

#[cfg(feature = "sprinklers_linux")]
use sprinklers_linux::LinuxGpioConfig;

use serde::{Deserialize, Serialize};
use std::sync::Arc;

#[derive(Debug, Serialize, Deserialize)]
#[serde(tag = "provider")]
pub enum ZoneInterfaceConfig {
    Mock {
        num_zones: ZoneNum,
    },
    #[cfg(feature = "sprinklers_linux")]
    LinuxGpio(LinuxGpioConfig),
}

impl Default for ZoneInterfaceConfig {
    fn default() -> Self {
        ZoneInterfaceConfig::Mock { num_zones: 6 }
    }
}

impl ZoneInterfaceConfig {
    pub fn build(self) -> eyre::Result<Arc<dyn ZoneInterface>> {
        Ok(match self {
            ZoneInterfaceConfig::Mock { num_zones } => Arc::new(MockZoneInterface::new(num_zones)),
            #[cfg(feature = "sprinklers_linux")]
            ZoneInterfaceConfig::LinuxGpio(config) => Arc::new(config.build()?),
        })
    }
}
